%main program for solving steady state 2D liquid electrolyte out of capillary
%calculate Z_n from Z'
%calculate from Z'(1) = 0 as boundary condition
%screen effect of phi_appl

%Import parameters from file
filename = 'parameters.txt';
M = readmatrix(filename);
Para = M(:,1);
for i = 1:length(Para)
    fprintf('%e\n', Para(i));
end

%Import R and theta grid
Grid_R = readmatrix('Grid_R.txt');
Grid_Y = readmatrix('Grid_Y.txt');
m = length(Grid_R);
m_e = find(Grid_R == 1);
n = length(Grid_Y);

%Read parameters
r_c = Para(1);  %radius of capillary, in m
h_c = Para(2);  %height of electrolyte in capillary, in m
gamma_surf = Para(3);  %surface tension of solution, in N/m
rho = Para(4);  %density of solution, in kg/m^3
d = Para(5);  %distance between opening of capillary and the sample, in m
phi_appl = Para(6);  %applied potential, in V

%Dimensionless conversion
d_n = d / r_c;  %distance normalized to radius of capillary, dimensionless

%Pressure coefficient calculation
co_H = 2 * gamma_surf / r_c;  %surface tension coefficient, in Pa
co_E = 8.85e-12 * phi_appl * phi_appl / (r_c * r_c);  %electric pressure coefficient, in Pa
co_g = rho * 9.8 * h_c;  %gravity pressure, in Pa
disp(co_H);
disp(co_E);
disp(co_g);

%Define initial values
phi_sol = 0.5 * ones(m,n);  %initial value of phi
Z_n_old = d_n * ones(1,m);  %initial value of Z_n
Z_n_new = zeros(1,m);
dZ_dR = zeros(1,m_e);  %initial value of dZ_dR = 0
G = zeros(1,m_e);  %G = (co_E * E_n * E_n + co_g) / co_H, dimensionless
it_max = 10000;
Z_err = d_n;  %initial value of Z_err
Convergence = 1;
for it = 1:it_max
    phi_ini = phi_sol;  %Take last phi_sol as initial value
    if Z_err > (d_n * 1e-6)
        Z_n_new = Z_n_old;  %Copy Z_n_old to Z_n_new
        [alpha beta gamma lama lamc co_xy] = Coefficient(Z_n_old, Grid_R, Grid_Y);  %Calculate coefficients from the geometry        
        phi_sol = ADI_liquid(alpha, beta, gamma, lama, lamc, co_xy, phi_appl, phi_ini, m, n, m_e, it_max, 1e-5);
        E_n = Flux_RZ(phi_sol, Z_n_old, Grid_R, dZ_dR, Grid_Y(n) - Grid_Y(n-1), m_e, n);       
        for i = 1:(m_e-1)
            G(i) = (co_E * E_n(i) * E_n(i) + co_g) / co_H;        
        end
        G_integ = 0;  %Initial value of G_integ = 0
        for i = 2:m_e
            G_integ = G_integ + (G(i) + G(i-1)) / 2 * (Grid_R(i) - Grid_R(i-1));
            if G_integ < 1
                dZ_dR(i) = sqrt(G_integ * G_integ / (1 - G_integ * G_integ));
            else
                Convergence = 0;
            end
        end
        if Convergence == 1
            for i = (m_e-1):(-1):1
                Z_n_new(i) = Z_n_new(i+1) - dZ_dR(i) * (Grid_R(i+1) - Grid_R(i));
            end
            figure(1)
            plot(Grid_R, Z_n_new);
            title(string(it));
            Z_err = max(abs(Z_n_new - Z_n_old));  %Calculate Z_err before updating Z_n_old
            disp(Z_err);
            Z_n_old = Z_n_new;  %Update Z_n_old
        else
            disp('Non convergent!');
            break;
        end
    else
        break;
    end
end

%Output Z_n_new
Z_out = fopen(strcat(string(r_c), '_', string(d_n), '_', string(phi_appl), '.txt'), 'w');
for i = 1:m_e
    fprintf(Z_out, '%f\t%f\n', Grid_R(i), Z_n_new(i));
end
fclose(Z_out);
    
%Plot phi map
R_map = zeros(m,n);
Z_map = zeros(m,n);
for i = 1:m
    for j = 1:n
        R_map(i,j) = Grid_R(i);
        Z_map(i,j) = Grid_Y(j) * Z_n_new(i);
    end
end
figure(2);  %Plot phi_out
pcolor(R_map, Z_map, phi_sol);
hold on;
shading interp;
colorbar;
colormap(jet);
axis equal;
xlabel('R');
ylabel('Z');
pause(1);


