%main program for solving steady state 2D liquid electrolyte out of capillary
%calculate Z_n from Z'
%calculate from Z'(1) = 0 as boundary condition
%screen effect of phi_appl for critical conditions
%Copy to the folder of Figure 1 for calling functions

%Import parameters from file
filename = 'parameters.txt';
M = readmatrix(filename);
Para = M(:,1);
for i = 1:length(Para)
    fprintf('%e\n', Para(i));
end

%Import R and theta grid
Grid_R = readmatrix('Grid_R.txt');
Grid_Y = readmatrix('Grid_Y.txt');
m = length(Grid_R);
m_e = find(Grid_R == 1);
n = length(Grid_Y);

%Read parameters
r_c = Para(1);  %radius of capillary, in m
h_c = Para(2);  %height of electrolyte in capillary, in m
gamma_surf = Para(3);  %surface tension of solution, in N/m
rho = Para(4);  %density of solution, in kg/m^3

%Dimensionless conversion
d_n = [1e-9/r_c 2e-9/r_c 5e-9/r_c 1e-8/r_c 2e-8/r_c 5e-8/r_c 1e-7/r_c 2e-7/r_c 5e-7/r_c 1e-6/r_c 2e-6/r_c 5e-6/r_c 1e-5/r_c];

%Pressure coefficient calculation
co_H = 2 * gamma_surf / r_c;  %surface tension coefficient, in Pa
co_g = rho * 9.8 * h_c;  %gravity pressure, in Pa

%Define initial values
Z_n_new = zeros(1,m);
G = zeros(1,m_e);  %G = (co_E * E_n * E_n + co_g) / co_H, dimensionless
it_max = 10000;
phi_c = zeros(1,length(d_n));  %critical value of phi

%Output phi_c
phi_c_out = fopen(strcat('phi_c vs d_n-', string(r_c), '-1003.txt'), 'w');
for d_it = 1:length(d_n)
    phi_max = 1e4;
    phi_min = 0;
    phi_err = 1e-3;
    while phi_max - phi_min > phi_err
        phi_rand = phi_min + (phi_max - phi_min) * rand;
%         fprintf('%f\t%f\t%f\n', phi_rand, phi_min, phi_max);  %display phi values
        co_E = 8.85e-12 * phi_rand * phi_rand / (r_c * r_c);  %electric pressure coefficient, in Pa
        Convergence = 1;
        %initial values
        phi_sol = 0.5 * ones(m,n);  %initial value of phi
        Z_n_old = d_n(d_it) * ones(1,m);  %initial value of Z_n
        dZ_dR = zeros(1,m_e);  %initial value of dZ_dR = 0
        Z_err = d_n(d_it);  %initial value of Z_err
        for it = 1:it_max
            phi_ini = phi_sol;  %Take last phi_sol as initial value
            if Z_err > (d_n(d_it) * 1e-6)
                Z_n_new = Z_n_old;  %Copy Z_n_old to Z_n_new so that the hard boundary is updated
                [alpha beta gamma lama lamc co_xy] = Coefficient(Z_n_old, Grid_R, Grid_Y);  %Calculate coefficients from the geometry        
                phi_sol = ADI_liquid(alpha, beta, gamma, lama, lamc, co_xy, phi_rand, phi_ini, m, n, m_e, it_max, 1e-5);
                E_n = Flux_RZ(phi_sol, Z_n_old, Grid_R, dZ_dR, Grid_Y(n) - Grid_Y(n-1), m_e, n); 
                for i = 1:(m_e-1)
                    G(i) = (co_E * E_n(i) * E_n(i) + co_g) / co_H;
                end
                G_integ = 0;  %Initial value of G_integ = 0
                for i = 2:m_e
                    G_integ = G_integ + (G(i) + G(i-1)) / 2 * (Grid_R(i) - Grid_R(i-1));
                    if G_integ < 1
                        dZ_dR(i) = sqrt(G_integ * G_integ / (1 - G_integ * G_integ));
                    else
                        Convergence = 0;
                    end
                end
                if Convergence == 1
                    for i = (m_e-1):(-1):1
                        Z_n_new(i) = Z_n_new(i+1) - dZ_dR(i) * (Grid_R(i+1) - Grid_R(i));
                    end
                    Z_err = max(abs(Z_n_new - Z_n_old));  %Calculate Z_err before updating Z_n_old
                    Z_n_old = Z_n_new;  %Update Z_n_old
                else
                    break;
                end
            else
                break;
            end
        end
        if Convergence == 1
            phi_min = phi_rand;
        else
            phi_max = phi_rand;
        end
    end
    phi_c(d_it) = phi_rand;
    fprintf('%e\t%e\n', d_n(d_it), phi_c(d_it));
    fprintf(phi_c_out, '%e\t%e\n', d_n(d_it), phi_c(d_it));
end
fclose(phi_c_out);

%Plot phi_c vs d_n
scatter(d_n, phi_c);



