%Dynamic deformation with constant potential for 1D gel deformation
%Approaching the probe at constant speed

%Parameters
eps = 8.85e-12;  %eps0, in F/m
phi_appl = 1;  %phi_appl, in V
a = 1e-5;  %thickness of gel, in m
E_Y = 1e4;  %elastic modulus of gel, in Pa
yeta = 1e3;  %viscous modulus of gel, in Pa*s
d0 = 2e-5;  %distance, in m
del_t = 1e-4;  %time step, in s
v = 1e-6;  %approach speed, in m/s
t_max = d0 / v;  %max time, in s

%Dimensionless conversion
d0_n = d0 / a;
v_n = v / a;  %in s^-1
A = eps * phi_appl * phi_appl * del_t / (a * a * yeta);
B = E_Y * del_t / yeta;
co_i = eps * phi_appl / a;  %in C/m^2

%Solve by implicit method
X_old = 0;
t = del_t:del_t:t_max;
k = length(t);
X = zeros(1,k);
dt_n = zeros(1,k);
current_density = zeros(1,k);  %in A/m^2
for i = 1:k
    fun = @(X_new) A / ((d0_n - v_n * t(i) - X_new) * (d0_n - v_n * t(i) - X_new)) - (1 + B) * X_new + X_old;
    X(i) = fzero(fun, X_old);  %Solve X(i) implicitly from X(i-1)
    dt_n(i) = d0_n - v_n * t(i) - X(i);  %Calculate distance
    current_density(i) = co_i / (dt_n(i) * dt_n(i)) * (v_n + (X(i) - X_old) / del_t);  %Calculate current
    X_old = X(i);
    if dt_n(i) < 0
        dt_n(i) = 0;
        touch = i;
        break;
    end
end

%Export distance and current density vs. time
result = fopen('1um per s.txt','w');
for i = 1:touch
    fprintf(result, '%e\t%e\t%e\n', t(i), dt_n(i), current_density(i));
end

%Plot distance for 1um per s
disp(t(touch));
disp(X(touch));
figure(1)
ax = gca;
ax.FontSize = 22;
set(gca, 'xlim', [19.55 19.8]);
set(gca, 'xtick', 19.55:0.05:19.8);
yyaxis left
plot(t(1:touch), dt_n(1:touch), 'LineWidth', 4);
xlabel('{\it t} / s', 'FontSize', 28);
ylabel('{\it d} / {\it a}, dimensionless', 'FontSize', 28);
set(gca, 'ylim', [0 0.05]);
set(gca, 'ytick', 0:0.01:0.05);
yyaxis right
plot(t(1:touch), current_density(1:touch), 'LineWidth', 4);
ylabel('{\it i}, A/m^2', 'FontSize', 28);
set(gca, 'ylim', [0 0.03]);
set(gca, 'ytick', 0:0.005:0.03);

